/*
 * Purpose: All modal dialogs for AddressManager
 * Author:  Frank Bu
 * Created: 2002
 */

// ==========================================================================
// headers, declarations, constants
// ==========================================================================

#include "wx/wx.h"

#include "wx/statline.h"
#include "wx/minifram.h"

#include "dialogs.h"

#include "wx/arrimpl.cpp"
WX_DEFINE_OBJARRAY(ColumnFieldArray);

// --------------------------------------------------------------------------
// constants
// --------------------------------------------------------------------------
extern char * addresses_xpm[];

// IDs for the controls and the menu commands
enum
{
    ID_LISTBOX = 1,
	ID_NEW_BUTTON,
	ID_RENAME_BUTTON,
	ID_UP_BUTTON,
	ID_REMOVE_BUTTON,
	ID_DOWN_BUTTON
};

// --------------------------------------------------------------------------
// event tables and other macros for wxWindows
// --------------------------------------------------------------------------

// Event tables
BEGIN_EVENT_TABLE(ColumnDialog, wxDialog)
    EVT_LISTBOX(ID_LISTBOX, ColumnDialog::OnListBox)
	EVT_BUTTON(ID_NEW_BUTTON, ColumnDialog::OnNewButton)
	EVT_BUTTON(ID_RENAME_BUTTON, ColumnDialog::OnRenameButton)
	EVT_BUTTON(ID_UP_BUTTON, ColumnDialog::OnUpButton)
	EVT_BUTTON(ID_REMOVE_BUTTON, ColumnDialog::OnRemoveButton)
	EVT_BUTTON(ID_DOWN_BUTTON, ColumnDialog::OnDownButton)
END_EVENT_TABLE()


// ==========================================================================
// implementation
// ==========================================================================

// --------------------------------------------------------------------------
// ColumnDialog
// --------------------------------------------------------------------------


ColumnField::ColumnField(wxString name, int oldIndex, int oldWidth)
{
	m_name = name;
	m_oldIndex = oldIndex;
	m_oldWidth = oldWidth;
}

bool ColumnField::isNew()
{
	return m_oldIndex == -1;
}

int ColumnField::getOldIndex()
{
	return m_oldIndex;
}

int ColumnField::getOldWidth()
{
	return m_oldWidth;
}

wxString ColumnField::getName()
{
	return m_name;
}

void ColumnField::setName(wxString name)
{
	m_name = name;
}


ColumnDialog::ColumnDialog(wxWindow *parent, ColumnFieldArray& columns)
                 : wxDialog(parent, -1,
                            _("Tabellenkonfiguration"),
                            wxDefaultPosition,
                            wxDefaultSize,
                            wxDEFAULT_DIALOG_STYLE | wxDIALOG_MODAL)
{
	m_columns = columns;
   
    // create and populate the list of columns
    m_listBox = new wxListBox( this, ID_LISTBOX,
        wxDefaultPosition,
        wxDefaultSize,
        0, NULL,
        wxLB_SINGLE | wxLB_NEEDED_SB | wxLB_HSCROLL );

	for (unsigned i = 0; i < m_columns.GetCount(); i++) {
		wxString name = m_columns[i].getName();
		m_listBox->Append(name, &m_columns[i]);
	}

	wxBoxSizer* topSizer = new wxBoxSizer(wxHORIZONTAL);
	topSizer->Add(new wxStaticText(this, -1, "Neue Tabellenspalte:"), 0, wxALIGN_CENTER | wxLEFT | wxTOP, 10);
	m_newColumnTextCtrl = new wxTextCtrl(this, -1);
	topSizer->Add(m_newColumnTextCtrl, 1, wxLEFT | wxTOP, 10);
	topSizer->Add(new wxButton(this, ID_NEW_BUTTON, "Hinzufgen"), 0, wxALIGN_RIGHT | wxLEFT | wxTOP | wxRIGHT, 10);

	wxBoxSizer* middleSizer = new wxBoxSizer(wxHORIZONTAL);
	middleSizer->Add(m_listBox, 0, wxLEFT | wxTOP | wxEXPAND, 10);

	wxBoxSizer* listButtonsSizer = new wxBoxSizer(wxVERTICAL);
	listButtonsSizer->Add(new wxButton(this, ID_UP_BUTTON, "/\\"), 0, wxLEFT | wxTOP | wxEXPAND, 10);
	listButtonsSizer->Add(new wxButton(this, ID_REMOVE_BUTTON, "Lschen"), 0, wxLEFT | wxTOP | wxEXPAND, 10);
	listButtonsSizer->Add(new wxButton(this, ID_DOWN_BUTTON, "\\/"), 0, wxLEFT | wxTOP | wxEXPAND, 10);

	middleSizer->Add(listButtonsSizer, 0, wxCENTRE);

    middleSizer->Add(CreateTextSizer(_("Geben Sie im Textfeld\n"
									    "'Neue Tabellenspalte' den\n"
										"Namen einer neuen\n"
										"Tabellenspalte ein und klicken\n"
										"auf 'Hinzufgen' wenn Sie eine\n"
										"neue Tabellenspalte erstellen\n"
										"mchten, oder whlen Sie in der\n"
										"Liste links eine bestehende\n"
										"Tabellenspalte aus, wenn Sie\n"
										"diese verschieben, lschen\n"
										"oder umbennen mchten.")),
                                  0, wxLEFT | wxTOP | wxRIGHT, 10);

	wxBoxSizer* bottomSizer = new wxBoxSizer(wxHORIZONTAL);
	bottomSizer->Add(new wxStaticText(this, -1, "Gewhlte Tabellenspalte:"), 0, wxALIGN_CENTER | wxLEFT | wxTOP, 10);
	m_editColumnTextCtrl = new wxTextCtrl(this, -1);
	m_editColumnTextCtrl->Enable(false);
	bottomSizer->Add(m_editColumnTextCtrl, 1, wxALIGN_CENTER | wxLEFT | wxTOP, 10);
	m_editButton = new wxButton(this, ID_RENAME_BUTTON, "Umbenennen");
	m_editButton->Enable(false);
	bottomSizer->Add(m_editButton, 0, wxALIGN_RIGHT | wxLEFT | wxTOP | wxRIGHT, 10);

	wxBoxSizer* okCancelSizer = new wxBoxSizer(wxHORIZONTAL);
	okCancelSizer->Add(new wxButton(this, wxID_OK, "OK"), 0, wxTOP, 10);
	okCancelSizer->Add(new wxButton(this, wxID_CANCEL, "Abbrechen"), 0, wxALL, 10);

	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	topsizer->Add(topSizer, 0, wxEXPAND);
	topsizer->Add(middleSizer, 1, wxEXPAND);
	topsizer->Add(bottomSizer, 0, wxEXPAND);
	topsizer->Add(okCancelSizer, 0, wxALIGN_RIGHT);

	SetAutoLayout(TRUE);     // tell dialog to use sizer
	SetSizer(topsizer);      // actually set the sizer

	topsizer->Fit(this);            // set size to minimum size as calculated by the sizer
	topsizer->SetSizeHints(this);   // set size hints to honour mininum size

    Centre(wxBOTH | wxCENTRE_ON_SCREEN);
}

ColumnDialog::~ColumnDialog()
{
	for (int i = 0; i < m_listBox->GetCount(); i++) {
		ColumnField* pField = (ColumnField*)m_listBox->GetClientData(i);
		if (pField->isNew()) delete pField;
	}
}

void ColumnDialog::OnListBox(wxCommandEvent& event)
{
    int sel = event.GetSelection();
    if (sel != -1) {
		m_editColumnTextCtrl->SetValue(m_listBox->GetString(sel));
		m_editColumnTextCtrl->Enable(true);
		m_editButton->Enable(true);
	}
}

void ColumnDialog::OnNewButton(wxCommandEvent &event)
{
	wxString name = m_newColumnTextCtrl->GetValue();
	name.Trim(true);
	name.Trim(false);
	if (name.Length() == 0) {
		wxMessageDialog dialog(this, "Der Name einer Tabellenspalte kann nicht leer sein.",
			"Address Manager", wxOK | wxICON_INFORMATION);
		dialog.ShowModal();
		return;
	}
	for (int i = 0; i < m_listBox->GetCount(); i++) {
		if (m_listBox->GetString(i) == name) {
			wxMessageDialog dialog(this, "Es existiert bereits eine Tabellenspalte gleichen Namens.",
				"Address Manager", wxOK | wxICON_INFORMATION);
			dialog.ShowModal();
			return;
		}
	}
	m_listBox->Append(name, new ColumnField(name));
	m_listBox->Select(m_listBox->GetCount() - 1);
	m_editColumnTextCtrl->SetValue(name);
	m_editColumnTextCtrl->Enable(true);
	m_editButton->Enable(true);
}

void ColumnDialog::OnRenameButton(wxCommandEvent &event)
{
    int sel = m_listBox->GetSelection();
    if (sel != -1) {
		wxString name = m_editColumnTextCtrl->GetValue();
		name.Trim(true);
		name.Trim(false);
		if (name.Length() == 0) {
			wxMessageDialog dialog(this, "Der Name einer Tabellenspalte kann nicht leer sein.",
				"Address Manager", wxOK | wxICON_INFORMATION);
			dialog.ShowModal();
			return;
		}
		for (int i = 0; i < m_listBox->GetCount(); i++) {
			if (i != sel && m_listBox->GetString(i) == name) {
				wxMessageDialog dialog(this, "Es existiert bereits eine Tabellenspalte gleichen Namens.",
					"Address Manager", wxOK | wxICON_INFORMATION);
				dialog.ShowModal();
				return;
			}
		}
		m_listBox->SetString(sel, name);
		ColumnField* pField = (ColumnField*) m_listBox->GetClientData(sel);
		pField->setName(name);
	}
}

void ColumnDialog::OnUpButton(wxCommandEvent &event)
{
    int sel = m_listBox->GetSelection();
    if (sel > 0) {
		void* pClientData = m_listBox->GetClientData(sel - 1);
		wxString name = m_listBox->GetString(sel - 1);
		m_listBox->SetClientData(sel - 1, m_listBox->GetClientData(sel));
		m_listBox->SetString(sel - 1, m_listBox->GetString(sel));
		m_listBox->SetClientData(sel, pClientData);
		m_listBox->SetString(sel, name);
		m_listBox->Select(sel - 1);
	}
}

void ColumnDialog::OnRemoveButton(wxCommandEvent &event)
{
    int sel = m_listBox->GetSelection();
    if (sel != -1) {
		if (m_listBox->GetCount() == 1) {
			wxMessageDialog dialog(this, "Es mu mindestens eine Tabellenspalte vorhanden sein.",
				"Address Manager", wxOK | wxICON_INFORMATION);
			dialog.ShowModal();
			return;
		}
		ColumnField* pField = (ColumnField*) m_listBox->GetClientData(sel);
		m_listBox->Delete(sel);
		m_editColumnTextCtrl->SetValue("");
		m_editColumnTextCtrl->Enable(false);
		m_editButton->Enable(false);
		if (pField->isNew()) delete pField;
	}
}

void ColumnDialog::OnDownButton(wxCommandEvent &event)
{
    int sel = m_listBox->GetSelection();
    if (sel != -1 && sel < m_listBox->GetCount() - 1) {
		void* pClientData = m_listBox->GetClientData(sel + 1);
		wxString name = m_listBox->GetString(sel + 1);
		m_listBox->SetClientData(sel + 1, m_listBox->GetClientData(sel));
		m_listBox->SetString(sel + 1, m_listBox->GetString(sel));
		m_listBox->SetClientData(sel, pClientData);
		m_listBox->SetString(sel, name);
		m_listBox->Select(sel + 1);
	}
}


ColumnFieldArray ColumnDialog::getNewColumns()
{
	ColumnFieldArray columns;
	for (int i = 0; i < m_listBox->GetCount(); i++) {
		columns.Add(*((ColumnField*)m_listBox->GetClientData(i)));
	}
	return columns;
}


// --------------------------------------------------------------------------
// SortDialog
// --------------------------------------------------------------------------

BEGIN_EVENT_TABLE(SortDialog, wxDialog)
    EVT_BUTTON(wxID_OK, SortDialog::OnOK)
END_EVENT_TABLE()


SortDialog::SortDialog(wxWindow *parent, ColumnFieldArray& columns)
                 : wxDialog(parent, -1,
                            _("Sortieren"),
                            wxDefaultPosition,
                            wxDefaultSize,
                            wxDEFAULT_DIALOG_STYLE | wxDIALOG_MODAL)
{
	m_columns = columns;
   
    // create and populate the list of columns
	m_sort1Choice = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
	m_sort2Choice = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);
	m_sort3Choice = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, 0, NULL);

	for (unsigned i = 0; i < m_columns.GetCount(); i++) {
		wxString name = m_columns[i].getName();
		m_sort1Choice->Append(name, &m_columns[i]);
		m_sort2Choice->Append(name, &m_columns[i]);
		m_sort3Choice->Append(name, &m_columns[i]);
	}
	m_sort1Choice->SetSelection(0);
	m_sort2Choice->SetSelection(0);
	m_sort3Choice->SetSelection(0);

	wxBoxSizer* sort1HeadSizer = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* sort1ContentSizer = new wxBoxSizer(wxHORIZONTAL);
	sort1HeadSizer->Add(new wxStaticText(this, -1, "Sortieren nach"), 0, wxCENTER | wxLEFT | wxTOP, 10);
	sort1HeadSizer->Add(new wxStaticLine(this, -1), 1, wxCENTER | wxLEFT | wxRIGHT | wxTOP, 10);
	sort1ContentSizer->Add(m_sort1Choice, 0, wxLEFT | wxTOP, 10);
	m_sort1CheckboxUp = new wxRadioButton(this, -1, "Aufsteigend", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
	m_sort1CheckboxDown = new wxRadioButton(this, -1, "Absteigend");
	wxBoxSizer* rb1Sizer = new wxBoxSizer(wxVERTICAL);
	rb1Sizer->Add(m_sort1CheckboxUp, 0, wxTOP, 5);
	rb1Sizer->Add(m_sort1CheckboxDown, 0, wxTOP, 5);
	sort1ContentSizer->Add(rb1Sizer, 0, wxLEFT | wxRIGHT, 10);

	wxBoxSizer* sort2HeadSizer = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* sort2ContentSizer = new wxBoxSizer(wxHORIZONTAL);
	sort2HeadSizer->Add(new wxStaticText(this, -1, "Anschlieend nach"), 0, wxCENTER | wxLEFT | wxTOP, 10);
	sort2HeadSizer->Add(new wxStaticLine(this, -1), 1, wxCENTER | wxLEFT | wxRIGHT | wxTOP, 10);
	sort2ContentSizer->Add(m_sort2Choice, 0, wxLEFT | wxTOP, 10);
	m_sort2CheckboxUp = new wxRadioButton(this, -1, "Aufsteigend", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
	m_sort2CheckboxDown = new wxRadioButton(this, -1, "Absteigend");
	wxBoxSizer* rb2Sizer = new wxBoxSizer(wxVERTICAL);
	rb2Sizer->Add(m_sort2CheckboxUp, 0, wxTOP, 5);
	rb2Sizer->Add(m_sort2CheckboxDown, 0, wxTOP, 5);
	sort2ContentSizer->Add(rb2Sizer, 0, wxLEFT | wxRIGHT, 10);

	wxBoxSizer* sort3HeadSizer = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer* sort3ContentSizer = new wxBoxSizer(wxHORIZONTAL);
	sort3HeadSizer->Add(new wxStaticText(this, -1, "Zuletzt nach"), 0, wxCENTER | wxLEFT | wxTOP, 10);
	sort3HeadSizer->Add(new wxStaticLine(this, -1), 1, wxCENTER | wxLEFT | wxRIGHT | wxTOP, 10);
	sort3ContentSizer->Add(m_sort3Choice, 0, wxLEFT | wxTOP, 10);
	m_sort3CheckboxUp = new wxRadioButton(this, -1, "Aufsteigend", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
	m_sort3CheckboxDown = new wxRadioButton(this, -1, "Absteigend");
	wxBoxSizer* rb3Sizer = new wxBoxSizer(wxVERTICAL);
	rb3Sizer->Add(m_sort3CheckboxUp, 0, wxTOP, 5);
	rb3Sizer->Add(m_sort3CheckboxDown, 0, wxTOP, 5);
	sort3ContentSizer->Add(rb3Sizer, 0, wxLEFT | wxRIGHT, 10);

	wxBoxSizer* okCancelSizer = new wxBoxSizer(wxHORIZONTAL);
	wxButton* okButton = new wxButton(this, wxID_OK, "OK");
	okButton->SetDefault();
	okCancelSizer->Add(okButton, 0, wxTOP, 10);
	okCancelSizer->Add(new wxButton(this, wxID_CANCEL, "Abbrechen"), 0, wxALL, 10);

	wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
	topsizer->Add(sort1HeadSizer, 0, wxEXPAND);
	topsizer->Add(sort1ContentSizer, 0, wxLEFT | wxRIGHT, 10);
	topsizer->Add(sort2HeadSizer, 0, wxEXPAND);
	topsizer->Add(sort2ContentSizer, 0, wxLEFT | wxRIGHT, 10);
	topsizer->Add(sort3HeadSizer, 0, wxEXPAND);
	topsizer->Add(sort3ContentSizer, 0, wxLEFT | wxRIGHT, 10);
	topsizer->Add(okCancelSizer, 0, wxTOP | wxALIGN_RIGHT, 10);

	SetAutoLayout(TRUE);     // tell dialog to use sizer
	SetSizer(topsizer);      // actually set the sizer

	topsizer->Fit(this);            // set size to minimum size as calculated by the sizer
	topsizer->SetSizeHints(this);   // set size hints to honour mininum size

    Centre(wxBOTH | wxCENTRE_ON_SCREEN);
}

ColumnFieldArray SortDialog::getSortOrder()
{
	ColumnFieldArray columns;
	int sel1 = m_sort1Choice->GetSelection();
	int sel2 = m_sort2Choice->GetSelection();
	int sel3 = m_sort3Choice->GetSelection();
	if (sel1 != 0) columns.Add(*((ColumnField*)m_sort1Choice->GetClientData(sel1)));
	if (sel2 != 0) columns.Add(*((ColumnField*)m_sort2Choice->GetClientData(sel2)));
	if (sel3 != 0) columns.Add(*((ColumnField*)m_sort3Choice->GetClientData(sel3)));
	return columns;
}

void SortDialog::OnOK(wxCommandEvent& event)
{
	int sel1 = m_sort1Choice->GetSelection();
	int sel2 = m_sort2Choice->GetSelection();
	int sel3 = m_sort3Choice->GetSelection();
	if (sel1 == 0 && sel2 == 0 && sel3 == 0) {
		wxMessageDialog dialog(this, "Bitte whlen Sie zumindest eine Tabellenspalte, nach der sortiert werden soll,\n"
			"oder brechen Sie die Auswahl mit Klick auf die Abbrechen-Schaltflche ab.",
			"Address Manager", wxOK | wxICON_INFORMATION);
		dialog.ShowModal();
		return;
	}
	if (sel2 != 0 && sel1 == 0) {
		wxMessageDialog dialog(this, "Bitte whlen Sie auch die Tabellenspalte aus, nach der zuerst sortiert wird,\n"
			"oder whlen Sie nur bei 'Sortieren nach' eine Tabellenspalte aus.",
			"Address Manager", wxOK | wxICON_INFORMATION);
		dialog.ShowModal();
		return;
	}
	if (sel3 != 0 && (sel1 == 0 || sel2 == 0)) {
		wxMessageDialog dialog(this, "Bitte whlen Sie auch die Tabellenspalte aus, nach der zuerst und als zweites sortiert wird,\n"
			"oder whlen Sie nur bei 'Sortieren nach' und / oder bei 'Anschlieend nach' eine Tabellenspalte aus.",
			"Address Manager", wxOK | wxICON_INFORMATION);
		dialog.ShowModal();
		return;
	}
	if ((sel1 != 0 && sel1 == sel2) || (sel1 != 0 && sel1 == sel3) || (sel2 != 0 && sel2 == sel3)) {
		wxMessageDialog dialog(this, "Bitte whlen Sie nur unterschiedliche Tabellenspalten oder '(keine)' aus.",
			"Address Manager", wxOK | wxICON_INFORMATION);
		dialog.ShowModal();
		return;
	}

	wxDialog::OnOK(event);
}


// --------------------------------------------------------------------------
// AddressManagerAboutDialog
// --------------------------------------------------------------------------

AddressManagerAboutDialog::AddressManagerAboutDialog(wxWindow *parent)
               : wxDialog(parent, -1,
                          _("ber Address Manager"),
                          wxDefaultPosition,
                          wxDefaultSize,
                          wxDEFAULT_DIALOG_STYLE | wxDIALOG_MODAL)
{
    // layout components
    wxBitmap bmp = wxBitmap(addresses_xpm);
    wxStaticBitmap *sbmp = new wxStaticBitmap(this, -1, bmp);
    wxBoxSizer *vertSizer = new wxBoxSizer( wxVERTICAL );
	wxBoxSizer *picAndTextSizer = new wxBoxSizer( wxHORIZONTAL );
    picAndTextSizer->Add( sbmp, 0, wxCENTRE | wxTOP | wxLEFT, 10);
    picAndTextSizer->Add( CreateTextSizer(_("Address Manager Version 1.0\n\n"
                                  "(c) 2002 Frank Bu, fb@frank-buss.de")),
                                  0, wxCENTRE | wxTOP | wxLEFT | wxRIGHT, 10);
    vertSizer->Add(picAndTextSizer, 0);
    vertSizer->Add(CreateButtonSizer(wxOK), 0, wxCENTRE | wxALL, 10);

    // activate
    SetSizer(vertSizer);
    SetAutoLayout(true);
    vertSizer->SetSizeHints(this);
    vertSizer->Fit(this);
    Centre(wxBOTH | wxCENTRE_ON_SCREEN);
}
